
#include "client_activity.h"
#include <string>
#include <thread>


namespace sample
{
    mdc::visual::PointCloud2 innoLidarFront1_pointCloud2Msg{};
    uint8_t const app_index = ClientActivity::LIDAR_INNO;
    uint8_t const inno_index = ClientActivity::LIDAR_INNO_IDX;


     void ClientActivity::InnoLidarFront1_ServiceAvailabilityCallback(ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles,
                                                      ara::com::FindServiceHandle handler)
     {
         log_.LogInfo() << "Enter Lidar Inno front 1 serviceAvailabilityCallback.";
         for (auto it : handles)
         {
             std::lock_guard<std::mutex> lock(proxySocAMutex_[app_index]);
             if (it.GetInstanceId().ToString() == "1" && proxySocA_[app_index] == nullptr)
             {
                 // 服务发现回调可能多次进入，因此proxy_为nullptr时才创建，避免proxy_重复创建导致未知问题
                 log_.LogInfo() << "Created proxy instance ";
                 proxySocA_[app_index] = std::make_shared<ProxySocA>(it);

                 proxySocA_[app_index]->LidarInnoPPC2Viz.SetReceiveHandler([this]() { ClientActivity:InnoLidarFront1_DataCallback(); });
                 const size_t maxSampleCount = 10U;
                 proxySocA_[app_index]->LidarInnoPPC2Viz.Subscribe(maxSampleCount);
             }
         }
     }

     void ClientActivity::InnoLidarFront1_setupFindService()
     {
     serviceHandleA[app_index] = ProxySocA::StartFindService(
         [this](ara::com::ServiceHandleContainer<ProxySocA::HandleType> handles, ara::com::FindServiceHandle handler)
         {
             ClientActivity::InnoLidarFront1_ServiceAvailabilityCallback(std::move(handles), handler);
         },
         ara::core::InstanceSpecifier(
             "visualization/visualization/LidarInnoRPort"));
     }

    void ClientActivity::InnoLidarFront1_DataCallback()
    {
        log_.LogInfo() << "Lidar Inno front 1 event received.";

        proxySocA_[app_index]->LidarInnoPPC2Viz.GetNewSamples([this](ara::com::SamplePtr<LidarPointCloud const> ptr)
                                                      {

            auto samplePC = ptr.Get();
    
            innoLidarFront1_pointCloud2Msg.header.stamp.sec = samplePC->header.stampLow;
            innoLidarFront1_pointCloud2Msg.header.stamp.nsec = samplePC->header.stampHigh;
            innoLidarFront1_pointCloud2Msg.width = samplePC->width;
            innoLidarFront1_pointCloud2Msg.data = samplePC->data;
            innoLidarFront1_pointCloud2Msg.header.frameId = samplePC->header.frameId;
            InnoLidarFront_pc2Queue[inno_index].push(innoLidarFront1_pointCloud2Msg);
    
            //innoPub_[inno_index].Publish(innoLidarFront1_pointCloud2Msg);
            log_.LogInfo() << "camera id: " << samplePC->width ; });
    }

    void ClientActivity::InnoLidarFront1_checkPc2Queue()
    {
        while (1)
        {
            if (!InnoLidarFront_pc2Queue[inno_index].empty())
            {
            	log_.LogInfo() << "Lidar Inno front 1 event sent.";
                innoPub_[inno_index].Publish(InnoLidarFront_pc2Queue[inno_index].front());
                InnoLidarFront_pc2Queue[inno_index].pop();
            }
            std::this_thread::sleep_for(std::chrono::milliseconds(10));
        }
    }

    void ClientActivity::InnoLidarFront1_runThread()
    {
    	InnoLidarFront_pc2Thread[inno_index]= std::thread(&ClientActivity::InnoLidarFront1_checkPc2Queue, this);
//        InnoLidarFront_pc2Thread[inno_index].join();
    }

    void ClientActivity::InnoLidarFront1_setupPublishers()
    {
        innoPub_[inno_index] = mdc::visual::Publisher::Advertise<mdc::visual::PointCloud2>("innovusion_lidar_front");
    }

     void ClientActivity::InnoLidarFront1_stop()
     {
         if (proxySocA_[app_index] != nullptr && proxySocA_[app_index]->LidarInnoPPC2Viz.IsSubscribed())
         {
         	proxySocA_[app_index]->LidarInnoPPC2Viz.UnsetReceiveHandler();
         	proxySocA_[app_index]->LidarInnoPPC2Viz.Unsubscribe();
         }

         ProxySocA::StopFindService(serviceHandleA[app_index]);
     }



        void ClientActivity::InnoLidarFront1_initPointCloud2()
    {
        int pointStep = 14;
        innoLidarFront1_pointCloud2Msg.header.frameId = frame_name;
        innoLidarFront1_pointCloud2Msg.height = 1;
        innoLidarFront1_pointCloud2Msg.width = 200000;
        innoLidarFront1_pointCloud2Msg.isBigendian = false;
        innoLidarFront1_pointCloud2Msg.pointStep = pointStep;
        innoLidarFront1_pointCloud2Msg.rowStep = pointStep * width;
        innoLidarFront1_pointCloud2Msg.isDense = true;
        innoLidarFront1_pointCloud2Msg.fields = InnoCommon_ConstructPointField();

        //innoPub_[inno_index].Advertise<mdc::visual::PointCloud2>("innovusion_lidar");
    }
};
